#include "Toy.h"
#include "cstring"
#include <utility>

Toy::Toy(
        const char* name,
        const char* date,
        const char*provider,
        double code,
        const char* type,
        double price,
        const char* range,
        int amount):
        m_code(code),
        m_price(price),
        m_amount(amount){
    strcpy(m_name, name);
    strcpy(m_date, date);
    strcpy(m_provider,provider);
    strcpy(m_type, type);
    strcpy(m_range, range);
}

std::istream &operator>>(std::istream &p_in, Toy &toy) {
    p_in.getline(toy.m_name, 128);
    p_in.getline(toy.m_date, 128);
    p_in.getline(toy.m_provider, 128);
    p_in.getline(toy.m_type, 128);
    p_in.getline(toy.m_range, 128);
    p_in>>toy.m_code
        >>toy.m_price
        >>toy.m_amount;
    return p_in;
}

std::ostream &operator<<(std::ostream &p_out, const Toy &toy) {
    p_out << toy.m_name << std::endl
          << toy.m_date << std::endl
          << toy.m_provider << std::endl
          << toy.m_code << std::endl
          << toy.m_type << std::endl
          << toy.m_price << std::endl
          << toy.m_range << std::endl
          << toy.m_amount << std::endl;
    return p_out;

}

// НЕЛЬЗЯ ПРИМЕНЯТЬ std::move К КОНСТАНТНОЙ ССЫЛКЕ,
// ВСЁ РАВНО КОПИРОВАНИЕ ПРОИЗОЙДЁТ
File::File(const std::string& fname): m_fname(std::move(fname)) {}

void File::create_file()
{
    std::ofstream fout(m_fname, std::ios::binary);

    int choice;
    Toy elem;
    std::cout << "Enter the information of new products" << std::endl;
    std::cout << "| Name | Date | Provider | Type | Range | Code | Price | Amount |" << std::endl;
    do {
        std::cin >> elem;
        fout.write(reinterpret_cast<char*>(&elem), sizeof(Toy));
        std::cout << "0 - End of file"     << std::endl
                  << "1 - Continue writting" << std::endl
                  << ">> ";
        std::cin >> choice;
        if (choice < 0 || choice > 1) {
            std::cout << "There's not such a request" << std::endl;
            break;
        }
    } while (choice == 1);
}

void File::add_to_file(const std::initializer_list<Toy> & list) {
    std::ofstream fout(m_fname, std::ios::binary | std::ios::app);
    for (const auto& elem : list)
        fout.write(reinterpret_cast<const char*>(&elem), sizeof(Toy));
}

void File::get_data() {
    std::ifstream fin(m_fname,std::ios::binary);
    Toy elem;
    int idx = 0;
    while (fin.read(reinterpret_cast<char*>(&elem), sizeof(Toy))) {
        idx++;
        m_arr.insert(std::make_pair(idx, elem));
    }
}

void File::delete_from_file(int index)
{
    get_data();
    m_arr.erase(index);
    std::ofstream fout(m_fname + "_new", std::ios::binary);
    for (const auto& elem : m_arr)
        fout.write(reinterpret_cast<const char*>(&elem.second), sizeof(Toy));
}

void File::replace_in_file(int index, const Toy & toy) {
    get_data();
    std::ofstream fout(m_fname + "_new",std::ios::binary);

    m_arr[index] = toy;
    for (const auto& elem : m_arr)
        fout.write(reinterpret_cast<const char*>(&elem.second), sizeof(Toy));
}

void File::transform_to_text() {
    get_data();
    std::ofstream fout(m_fname + ".txt");
    fout << std::setw(72) << "Toys "<<std::endl;
    fout << "|" << std::setw(4) << "ID "<<"|"
         << std::setw(18) << "Name" << "|"
         << std::setw(18) << "Date" << "|"
         << std::setw(18) << "Provider" << "|"
         << std::setw(18) << "Code" << "|"
         << std::setw(18) << "Type" << "|"
         << std::setw(18) << "Price" << "|"
         << std::setw(18) << "Range" << "|"
         << std::setw(18) << "Amount" << "|"
         << std::endl;size_t count = 0;
    for (const auto& elem : m_arr) {
        count++;
        fout << "|" << std::setw(4) << count << "|"
             << std::setw(18) << elem.second.m_name << "|"
             << std::setw(18) << elem.second.m_date << "|"
             << std::setw(18) << elem.second.m_provider << "|"
             << std::setw(18) << elem.second.m_code << "|"
             << std::setw(18) << elem.second.m_type << "|"
             << std::setw(18) << elem.second.m_price << "|"
             << std::setw(18) << elem.second.m_range << "|"
             << std::setw(18) << elem.second.m_amount << "|"
             << std::endl;
    }
}

void File::search_by_date(const std::string & date) {
    get_data();
    std::ofstream fout(date, std::ios::binary);
    for (const auto& elem : m_arr) {
        if (elem.second.m_date == date)
            fout.write(reinterpret_cast<const char*>(&elem.second), sizeof(Toy));
    }
}

void File::search_by_provider(const std::string &provider) {
    get_data();
    std::ofstream fout(provider, std::ios::binary);
    for (const auto& elem : m_arr) {
        if (elem.second.m_provider == provider)
            fout.write(reinterpret_cast<const char*>(&elem.second), sizeof(Toy));
    }
}

void File::search_by_price(double price) {
    get_data();
    std::ofstream fout(std::to_string(price), std::ios::binary);
    for (const auto& elem : m_arr) {
        if (elem.second.m_price == price)
            fout.write(reinterpret_cast<const char*>(&elem.second), sizeof(Toy));
    }


}
